@@ -24,6 +24,7 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include <QPainter>
 #include <QPen>
 #include <QImage>
+#include <QFontMetrics>
 #include <iostream>
 
 #include "expose.h"
@@ -47,6 +48,7 @@ Canvas::Canvas(QWidget *parent)
       bDisplayTimeSeries(true),
       bDisplayLearned(true),
       bDisplayGrid(true),
+      bDisplayLegend(true),
       crosshair(QPainterPath()),
       bShowCrosshair(false),
       bNewCrosshair(true),
@@ -83,7 +85,7 @@ Canvas::Canvas(QWidget *parent)
 
 Canvas::~Canvas()
 {
-    if(data) DEL(data);
+    DEL(data);
 }
 
 void Canvas::dragEnterEvent(QDragEnterEvent *event)
@@ -98,6 +100,7 @@ void Canvas::dropEvent(QDropEvent *event)
         QPointF position = event->pos();
         //qDebug() << "Dropping Target at coordinates: " << position;
         targets.push_back(toSampleCoords(position.x(), position.y()));
+        targetAge.push_back(0);
     }
     else if(event->mimeData()->text() == "Gaussian")
     {
@@ -125,6 +128,12 @@ void Canvas::SetModelImage(QImage image)
     repaint();
 }
 
+void Canvas::SetAnimationImage(QImage animation)
+{
+    maps.animation = QPixmap::fromImage(animation);
+    repaint();
+}
+
 void Canvas::SetCanvasType(int type)
 {
     if(canvasType || type)
@@ -159,18 +168,23 @@ void Canvas::PaintStandard(QPainter &painter, bool bSvg)
         DrawRewards();
         if(!maps.reward.isNull())
         {
-            painter.setBackgroundMode(Qt::TransparentMode);
+            painter.setBackgroundMode(Qt::OpaqueMode);
             painter.drawPixmap(geometry(), maps.reward);
         }
-        if(bSvg) DrawSamples(painter);
+        if(bSvg)
+        {
+            painter.setBackgroundMode(Qt::TransparentMode);
+            DrawSamples(painter);
+            DrawObstacles(painter);
+        }
         else
         {
             DrawSamples();
             painter.setBackgroundMode(Qt::TransparentMode);
             painter.drawPixmap(geometry(), maps.samples);
+            DrawObstacles();
+            painter.drawPixmap(geometry(), maps.obstacles);
         }
-        DrawObstacles(painter);
-        //painter.drawPixmap(geometry(), maps.obstacles);
     }
     if(bDisplayTrajectories)
     {
@@ -206,9 +220,9 @@ void Canvas::PaintStandard(QPainter &painter, bool bSvg)
             int w = width();
             int h = height();
             maps.model = QPixmap(w,h);
-            QBitmap bitmap(w,h);
-            bitmap.clear();
-            maps.model.setMask(bitmap);
+            //QBitmap bitmap(w,h);
+            //bitmap.clear();
+            //maps.model.setMask(bitmap);
             maps.model.fill(Qt::transparent);
             QPainter painter(&maps.model);
             DrawSampleColors(painter);
@@ -216,6 +230,11 @@ void Canvas::PaintStandard(QPainter &painter, bool bSvg)
         painter.setBackgroundMode(Qt::TransparentMode);
         painter.drawPixmap(geometry(), maps.model);
     }
+    if(!maps.animation.isNull())
+    {
+        painter.setBackgroundMode(Qt::TransparentMode);
+        painter.drawPixmap(geometry(), maps.animation);
+    }
     if(!bSvg && bDisplayInfo && !maps.info.isNull())
     {
         painter.setBackgroundMode(Qt::TransparentMode);
@@ -232,6 +251,8 @@ void Canvas::PaintStandard(QPainter &painter, bool bSvg)
     {
         if(bSvg)
         {
+            painter.setBackgroundMode(Qt::TransparentMode);
+            DrawAxes(painter);
         }
         else
         {
@@ -240,6 +261,11 @@ void Canvas::PaintStandard(QPainter &painter, bool bSvg)
             painter.drawPixmap(geometry(), maps.grid);
         }
     }
+    if(bDisplayLegend)
+    {
+        painter.setBackgroundMode(Qt::TransparentMode);
+        DrawLegend(painter);
+    }
 }
 
 void Canvas::PaintMultivariate(QPainter &painter, int type)
@@ -258,9 +284,9 @@ void Canvas::PaintMultivariate(QPainter &painter, int type)
             int w = width();
             int h = height();
             maps.samples = QPixmap(w,h);
-            QBitmap bitmap(w,h);
-            bitmap.clear();
-            maps.samples.setMask(bitmap);
+            //QBitmap bitmap(w,h);
+            //bitmap.clear();
+            //maps.samples.setMask(bitmap);
             maps.samples.fill(Qt::transparent);
             Expose::DrawData(maps.samples, data->GetSamples(), data->GetLabels(), data->GetFlags(), type, data->bProjected, dimNames, bounds);
         }
@@ -274,9 +300,9 @@ void Canvas::PaintMultivariate(QPainter &painter, int type)
             int w = width();
             int h = height();
             maps.trajectories = QPixmap(w,h);
-            QBitmap bitmap(w,h);
-            bitmap.clear();
-            maps.trajectories.setMask(bitmap);
+            //QBitmap bitmap(w,h);
+            //bitmap.clear();
+            //maps.trajectories.setMask(bitmap);
             maps.trajectories.fill(Qt::transparent);
             Expose::DrawTrajectories(maps.trajectories, data->GetTrajectories(trajectoryResampleType, trajectoryResampleCount, trajectoryCenterType, 0.1, true), data->GetLabels(), type, 0, bounds);
         }
@@ -290,9 +316,9 @@ void Canvas::PaintMultivariate(QPainter &painter, int type)
             int w = width();
             int h = height();
             maps.model = QPixmap(w,h);
-            QBitmap bitmap(w,h);
-            bitmap.clear();
-            maps.model.setMask(bitmap);
+            //QBitmap bitmap(w,h);
+            //bitmap.clear();
+            //maps.model.setMask(bitmap);
             maps.model.fill(Qt::transparent);
             Expose::DrawData(maps.model, data->GetSamples(), sampleColors, data->GetFlags(), type, data->bProjected, true, dimNames);
         }
@@ -325,9 +351,9 @@ void Canvas::PaintVariable(QPainter &painter, int type, fvec params)
         int w = width();
         int h = height();
         maps.samples = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.samples.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.samples.setMask(bitmap);
         maps.samples.fill(Qt::transparent);
         Expose::DrawVariableData(maps.samples, data->GetSamples(), data->GetLabels(), type, params, data->bProjected);
     }
@@ -339,9 +365,9 @@ void Canvas::PaintVariable(QPainter &painter, int type, fvec params)
         int w = width();
         int h = height();
         maps.trajectories = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.trajectories.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.trajectories.setMask(bitmap);
         maps.trajectories.fill(Qt::transparent);
     }
     painter.setBackgroundMode(Qt::TransparentMode);
@@ -352,9 +378,9 @@ void Canvas::PaintVariable(QPainter &painter, int type, fvec params)
         int w = width();
         int h = height();
         maps.model = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.model.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.model.setMask(bitmap);
         maps.model.fill(Qt::transparent);
         Expose::DrawVariableData(maps.model, data->GetSamples(), sampleColors, type, params, data->bProjected);
     }
@@ -367,17 +393,18 @@ void Canvas::paintEvent(QPaintEvent *event)
     if(bDrawing) return;
     bDrawing = true;
     QPainter painter(this);
-    if(!canvasType) PaintStandard(painter);
-    else if(canvasType <= 4) PaintMultivariate(painter, canvasType-1);
+    if(!canvasType) PaintStandard(painter); // we only draw if we're actually on the canvas
+    /*
+    else if(canvasType <= 5) PaintMultivariate(painter, canvasType-2); // 0: standard, 1: 3d, so we take out 2
     else
     {
         fvec params;
         params.push_back(xIndex);
         params.push_back(yIndex);
         params.push_back(zIndex);
-        PaintVariable(painter, canvasType-5, params);
+        PaintVariable(painter, canvasType-6, params);
     }
-
+    */
     bDrawing = false;
 }
 
@@ -389,6 +416,7 @@ QPointF Canvas::toCanvasCoords(fvec sample)
     sample -= center;
     QPointF point(sample[xIndex]*(zoom*zooms[xIndex]*height()),sample[yIndex]*(zoom*zooms[yIndex]*height()));
     point += QPointF(width()/2, height()/2);
+    point.setY(height()-point.y());
     return point;
 }
 
@@ -397,6 +425,7 @@ QPointF Canvas::toCanvas(fVec sample)
     sample -= center;
     QPointF point(sample[xIndex]*(zoom*zooms[xIndex]*height()),sample[yIndex]*(zoom*zooms[yIndex]*height()));
     point += QPointF(width()/2, height()/2);
+    point.setY(height()-point.y());
     return point;
 }
 
@@ -406,6 +435,7 @@ QPointF Canvas::toCanvasCoords(float x, float y)
     y -= center[yIndex];
     QPointF point(x*(zoom*zooms[xIndex]*height()),y*(zoom*zooms[yIndex]*height()));
     point += QPointF(width()/2, height()/2);
+    point.setY(height() - point.y());
     return point;
 }
 
@@ -413,6 +443,7 @@ fvec Canvas::fromCanvas(QPointF point)
 {
     int dim = data->GetDimCount();
     fvec sample(dim);
+    point.setY(height()-point.y());
     point -= QPointF(width()/2.f,height()/2.f);
     sample[xIndex] = point.x()/(zoom*zooms[xIndex]*height());
     sample[yIndex] = point.y()/(zoom*zooms[yIndex]*height());
@@ -422,8 +453,10 @@ fvec Canvas::fromCanvas(QPointF point)
 
 fvec Canvas::fromCanvas(float x, float y)
 {
+    if(!data) return fvec(2,0);
     int dim = data->GetDimCount();
     fvec sample(dim);
+    y = height() - y;
     x -= width()/2.f;
     y -= height()/2.f;
     sample[xIndex] = x/(zoom*zooms[xIndex]*height());
@@ -436,6 +469,7 @@ fvec Canvas::toSampleCoords(QPointF point)
 {
     int dim = data->GetDimCount();
     fvec sample(dim);
+    point.setY(height() - point.y());
     point -= QPointF(width()/2.f,height()/2.f);
     sample[xIndex] = point.x()/(zoom*zooms[xIndex]*height());
     sample[yIndex] = point.y()/(zoom*zooms[yIndex]*height());
@@ -447,6 +481,7 @@ fvec Canvas::toSampleCoords(float x, float y)
 {
     int dim = data->GetDimCount();
     fvec sample(dim);
+    y = height() - y;
     x -= width()/2.f;
     y -= height()/2.f;
     sample[xIndex] = x/(zoom*zooms[xIndex]*height());
@@ -457,12 +492,12 @@ fvec Canvas::toSampleCoords(float x, float y)
 
 fvec Canvas::canvasTopLeft()
 {
-    return toSampleCoords(0,0);
+    return toSampleCoords(0,height()-1);
 }
 
 fvec Canvas::canvasBottomRight()
 {
-    return toSampleCoords(width()-1,height()-1);
+    return toSampleCoords(width()-1,0);
 }
 
 QRectF Canvas::canvasRect()
@@ -549,17 +584,34 @@ void Canvas::FitToData()
     if(!data->GetCount() && !data->GetTimeSeries().size())
     {
         center = fvec(2,0);
+        zooms = fvec(2, 1.f);
         SetZoom(1);
         //qDebug() << "nothing to fit";
         return;
     }
     int dim = data->GetDimCount();
     center = fvec(dim,0);
+    if(data->GetCount() == 1)
+    {
+        center = data->GetSample(0);
+        zooms = fvec(dim,1.f);
+        SetZoom(1);
+        return;
+    }
     //qDebug() << "fit to data, dim: " << dim;
 
     // we go through all the data and find the boundaries
     std::pair<fvec,fvec> bounds = data->GetBounds();
     fvec mins = bounds.first, maxes = bounds.second;
+    FOR(d, mins.size())
+    {
+        if(maxes[d] - mins[d] > 1e6)
+        {
+            mins[d] = 0;
+            maxes[d] = 1;
+        }
+    }
+
     vector<fvec> samples = data->GetSamples();
 
     vector<TimeSerie>& series = data->GetTimeSeries();
@@ -580,6 +632,15 @@ void Canvas::FitToData()
         }
     }
     fvec diff = maxes - mins;
+    // we add 10% to the edges
+    mins -= diff*.05f;
+    maxes += diff*.05f;
+    diff = maxes - mins;
+
+    FOR(d, diff.size())
+    {
+        if(diff[d] == 0) diff[d] = 1e-6;
+    }
 
     center = mins + diff/2;
 
@@ -596,6 +657,86 @@ void Canvas::FitToData()
     SetZoom(1.f);
 }
 
+void Canvas::DrawLegend(QPainter &painter)
+{
+    int w = painter.viewport().width(), h = painter.viewport().height();
+    QFont font = painter.font();
+    font.setPointSize(10);
+    painter.setFont(font);
+
+    // we draw the reward colorbar
+    if(!maps.reward.isNull())
+    {
+        // we draw the colorbar
+        painter.setRenderHint(QPainter::Antialiasing, false);
+        painter.setOpacity(0.8);
+        int barW = 20;
+        QRect rect(w - barW - 32, 40, barW, 256);
+        painter.setBrush(Qt::NoBrush);
+        for(int i=0; i<rect.height(); i++)
+        {
+            float v = (1.f - i/(float)rect.height())*255.f;
+            v = max(0.f, min(255.f, v));
+            painter.setPen(QColor(255,255-v,255-v));
+            painter.drawLine(rect.x(), rect.y() + i, rect.x() + rect.width(), rect.y() + i);
+        }
+
+        // we draw the values on the colorbar
+        painter.setOpacity(1);
+        int steps = 4;
+        float vmax = 1.0;
+        float vmin = 0.0;
+        for(int i=0; i<steps+1; i++)
+        {
+            float v = (1.f - i/(float)steps);
+            QString text = QString("%1").arg(v*(vmax-vmin) + vmin, 0, 'f', 2);
+            int y = rect.y() + i*rect.height()/steps;
+            QRect textRect = QRect(rect.x()-40, y - 10, 40-6, 20);
+            painter.setPen(Qt::black);
+            painter.drawText(textRect, Qt::AlignRight + Qt::AlignVCenter, text);
+            painter.drawLine(rect.x(), y, rect.x()-4, y);
+        }
+        painter.setPen(QPen(Qt::black, 1));
+        painter.drawRect(rect);
+    }
+    else // we draw the samples legend
+    {
+        if(!data->GetCount()) return;
+        std::map<int,bool> labelList;
+        ivec labels = data->GetLabels();
+        FOR(i, labels.size())
+        {
+            labelList[labels[i]] = true;
+        }
+        painter.setPen(QPen(Qt::black, 1));
+        // we need to know the size of the legend rectangle
+        int rectWidth = 0;
+        QFontMetrics fm = painter.fontMetrics();
+        for(map<int,bool>::iterator it=labelList.begin(); it != labelList.end(); it++)
+        {
+            QString className = GetClassString(it->first);
+            QRect rect = fm.boundingRect(className);
+            rectWidth = max(rectWidth, rect.width());
+        }
+        rectWidth += 10; // we add the sample size;
+
+        int x = w - rectWidth - 40, y = 40;
+        painter.setRenderHint(QPainter::Antialiasing, false);
+        painter.drawRect(x-10,y-10, rectWidth+12, 20*labelList.size());
+        painter.setRenderHint(QPainter::Antialiasing, true);
+        for(map<int,bool>::iterator it=labelList.begin(); it != labelList.end(); it++)
+        {
+            int label = it->first;
+            QPointF point(x, y);
+            drawSample(painter, point, 10, label);
+            QString className = GetClassString(label);
+            painter.drawText(x + 8, point.y()+3, className);
+            //painter.drawText(QRect(x + 4, point.y()-10, 70, 20), Qt::AlignLeft + Qt::AlignCenter, className);
+            y += 20;
+        }
+    }
+}
+
 void Canvas::DrawAxes(QPainter &painter)
 {
     int w = width();
@@ -604,7 +745,9 @@ void Canvas::DrawAxes(QPainter &painter)
     QRectF bounding = canvasRect();
     // we round up the size to the closest decimal
     float scale = bounding.height();
-    if(scale <= 1e-10) return;
+    float scaleRatio = scale / bounding.width();
+    if(scaleRatio > 1000 || 1.f/scaleRatio > 1000) scale = (bounding.height() + bounding.width()) / 2;
+    if(scale <= 1e-5) return;
     float mult = 1;
     if(scale > 10)
     {
@@ -617,26 +760,150 @@ void Canvas::DrawAxes(QPainter &painter)
     if(mult == 0) mult = 1;
 
     // we now have the measure of the ticks, we can draw this
-    painter.setRenderHint(QPainter::TextAntialiasing);
+    painter.setBackgroundMode(Qt::TransparentMode);
+    painter.setRenderHint(QPainter::Antialiasing, true);
+    //painter.setRenderHint(QPainter::TextAntialiasing);
+    painter.setBrush(Qt::NoBrush);
     painter.setFont(QFont("Lucida Grande", 9));
-    for(float x = (int)(bounding.x()/mult)*mult; x < bounding.x() + bounding.width(); x += mult)
+    painter.setPen(QPen(Qt::black, 0.5, Qt::DotLine));
+    // we draw the grid lines
+    int minGridWidth = 32;
+    if(!data->bProjected && data->IsCategorical(xIndex))
     {
-        float canvasX = toCanvasCoords(x,0).x();
-        if(canvasX < 0 || canvasX > w) continue;
-        painter.setPen(QPen(Qt::black, 0.5, Qt::DotLine));
-        painter.drawLine(canvasX, 0, canvasX, h);
-        painter.setPen(QPen(Qt::black, 0.5));
-        painter.drawText(canvasX, h-5, QString("%1").arg((int)(x/mult)*mult));
+        int cnt = data->categorical[xIndex].size();
+        FOR(i, cnt)
+        {
+            float canvasX = toCanvasCoords(i,0).x();
+            if(canvasX < 0 || canvasX > w) continue;
+            painter.drawLine(canvasX, 0, canvasX, h);
+        }
+    }
+    else
+    {
+        int cnt = 0;
+        if ( bounding.width() < 0 ) {
+            for(float x = (int)(bounding.x()/mult)*mult; x > bounding.x() + bounding.width(); x -= mult) cnt++;
+        } else {
+            for(float x = (int)(bounding.x()/mult)*mult; x < bounding.x() + bounding.width(); x += mult) cnt++;
+        }
+        if(!cnt) mult = minGridWidth/(float)w;
+        else if(w/cnt < minGridWidth) mult *= (float)minGridWidth*cnt/w;
+        for(float x = (int)(bounding.x()/mult)*mult; x < bounding.x() + bounding.width(); x += mult)
+        {
+            float canvasX = toCanvasCoords(x,0).x();
+            if(canvasX < 0 || canvasX > w) continue;
+            painter.drawLine(canvasX, 0, canvasX, h);
+        }
+    }
+    painter.setPen(QPen(Qt::black, 0.5, Qt::DotLine));
+
+    if(!data->bProjected && data->IsCategorical(yIndex))
+    {
+        int cnt = data->categorical[yIndex].size();
+        FOR(i, cnt)
+        {
+            float canvasY = toCanvasCoords(0,i).y();
+            if(canvasY < 0 || canvasY > w) continue;
+            painter.drawLine(0, canvasY, w, canvasY);
+        }
+    }
+    else
+    {
+        int cnt = 0;
+        if ( bounding.width() < 0 ) {
+            for(float y = (int)(bounding.y()/mult)*mult; y > bounding.y() + bounding.height(); y -= mult) cnt++;
+        } else {
+            for(float y = (int)(bounding.y()/mult)*mult; y < bounding.y() + bounding.height(); y += mult) cnt++;
+        }
+        if(!cnt) mult = minGridWidth/(float)w;
+        if(w/cnt < minGridWidth) mult *= (float)minGridWidth*cnt/w;
+        for(float y = (int)(bounding.y()/mult)*mult; y < bounding.y() + bounding.height(); y += mult)
+        {
+            float canvasY = toCanvasCoords(0,y).y();
+            if(canvasY < 0 || canvasY > w) continue;
+            painter.drawLine(0, canvasY, w, canvasY);
+        }
+    }
+    // we draw the tick values
+    painter.setPen(QPen(Qt::black, 0.5));
+    if(!data->bProjected && data->IsCategorical(xIndex))
+    {
+        int cnt = data->categorical[xIndex].size();
+        FOR(i, cnt)
+        {
+            string name = data->GetCategorical(xIndex, i);
+            float canvasX = toCanvasCoords(i,0).x();
+            if(canvasX < 0 || canvasX > w) continue;
+            painter.drawText(canvasX, h-5, QString(name.c_str()));
+        }
+    }
+    else
+    {
+        float gridW = 0;
+        for(float x = (int)(bounding.x()/mult)*mult; x < bounding.x() + bounding.width(); x += mult)
+        {
+            float canvasX = toCanvasCoords(x,0).x();
+            if(gridW == 0)
+            {
+                float x2 = toCanvasCoords(x + mult, 0).x();
+                gridW = x2 - canvasX;
+            }
+            if(canvasX < 0 || canvasX > w) continue;
+            float val = (int)(x/mult)*mult;
+            QString s;
+            if(mult >= 1) s = QString("%1").arg(val, 0, 'f', 0);
+            else if(mult >= 0.1) s = QString("%1").arg(val, 0, 'f', 1);
+            else if(mult >= 0.01) s = QString("%1").arg(val, 0, 'f', 2);
+            else s = QString("%1").arg(val);
+            painter.drawText(canvasX, h-15, max((float)minGridWidth,gridW), 10, Qt::AlignLeft | Qt::AlignBottom, s);
+        }
     }
     // we now have the measure of the ticks, we can draw this
-    for(float y = (int)(bounding.y()/mult)*mult; y < bounding.y() + bounding.height(); y += mult)
+    painter.setPen(QPen(Qt::black, 0.5));
+    if(!data->bProjected && data->IsCategorical(yIndex))
     {
-        float canvasY = toCanvasCoords(0,y).y();
-        if(canvasY < 0 || canvasY > w) continue;
-        painter.setPen(QPen(Qt::black, 0.5, Qt::DotLine));
-        painter.drawLine(0, canvasY, w, canvasY);
+        int cnt = data->categorical[yIndex].size();
+        FOR(i, cnt)
+        {
+            string name = data->GetCategorical(yIndex, i);
+            float canvasY = toCanvasCoords(0,i).y();
+            if(canvasY < 0 || canvasY > w) continue;
+            painter.drawText(2, canvasY, QString(name.c_str()));
+        }
+    }
+    else
+    {
+        for(float y = (int)(bounding.y()/mult)*mult; y < bounding.y() + bounding.height(); y += mult)
+        {
+            float canvasY = toCanvasCoords(0,y).y();
+            if(canvasY < 0 || canvasY > w) continue;
+            float val = (int)(y/mult)*mult;
+            QString s;
+            if(mult >= 1) s = QString("%1").arg(val, 0, 'f', 0);
+            else if(mult >= 0.1) s = QString("%1").arg(val, 0, 'f', 1);
+            else if(mult >= 0.01) s = QString("%1").arg(val, 0, 'f', 2);
+            else s = QString("%1").arg(val);
+            painter.drawText(2, canvasY, s);
+        }
+    }
+
+    // we get the dimension names
+    QFont font = painter.font();
+    font.setPointSize(12);
+    painter.setFont(font);
+    if(xIndex < dimNames.size())
+    {
+        QString xlabel = dimNames[xIndex];
         painter.setPen(QPen(Qt::black, 0.5));
-        painter.drawText(2, canvasY, QString("%1").arg((int)(y/mult)*mult));
+        painter.drawText(w/2 - 100, h + 10, 200, 10, Qt::AlignTop | Qt::AlignHCenter, xlabel);
+    }
+    if(yIndex < dimNames.size())
+    {
+        QString ylabel = dimNames[yIndex];
+        painter.setPen(QPen(Qt::black, 0.5));
+        painter.rotate(-90);
+        painter.drawText(-h/2-100, -20, 200, 10, Qt::AlignTop | Qt::AlignHCenter, ylabel);
+        painter.rotate(90);
     }
 }
 
@@ -645,9 +912,9 @@ void Canvas::RedrawAxes()
     int w = width();
     int h = height();
     maps.grid = QPixmap(w,h);
-    QBitmap bitmap(w,h);
-    bitmap.clear();
-    maps.grid.setMask(bitmap);
+    //QBitmap bitmap(w,h);
+    //bitmap.clear();
+    //maps.grid.setMask(bitmap);
     maps.grid.fill(Qt::transparent);
 
     QPainter painter(&maps.grid);
@@ -692,9 +959,9 @@ void Canvas::DrawSamples()
         int w = width();
         int h = height();
         maps.samples = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.samples.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.samples.setMask(bitmap);
         maps.samples.fill(Qt::transparent);
         drawnSamples = 0;
         return;
@@ -707,11 +974,14 @@ void Canvas::DrawSamples()
         int w = width();
         int h = height();
         maps.samples = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.samples.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.samples.setMask(bitmap);
         maps.samples.fill(Qt::transparent);
         drawnSamples = 0;
+        //maps.model = QPixmap(w,h);
+        //maps.model.setMask(bitmap);
+        //maps.model.fill(Qt::transparent);
     }
     QPainter painter(&maps.samples);
     painter.setRenderHint(QPainter::Antialiasing, true);
@@ -761,20 +1031,18 @@ QPainterPath Canvas::DrawObstacle(Obstacle o)
     {
         float X, Y;
         X = aX * cosf(theta);
-        //Y = aY * sinf(theta);
         Y = aY * (theta>=0?1.f:-1.f) * powf((1-powf(cosf(theta),2.f*pX)),1./(2*pY));
 
         float RX = + X * cosf(angle) - Y * sinf(angle);
         float RY = + X * sinf(angle) + Y * cosf(angle);
 
-        point = QPointF(RX*(zoom*zooms[xIndex]*height()),RY*(zoom*zooms[yIndex]*height()));
-        if(theta==-PIf)
-        {
+        point = QPointF(RX*(zoom*zooms[xIndex]*height()),-RY*(zoom*zooms[yIndex]*height()));
+        if (theta==-PIf) {
             firstPoint = point;
             obstaclePath.moveTo(point);
-            continue;
+        } else {
+            obstaclePath.lineTo(point);
         }
-        obstaclePath.lineTo(point);
     }
     obstaclePath.lineTo(firstPoint);
     return obstaclePath;
@@ -818,9 +1086,9 @@ void Canvas::DrawObstacles()
     int w = width();
     int h = height();
     maps.obstacles = QPixmap(w,h);
-    QBitmap bitmap(w,h);
-    bitmap.clear();
-    maps.obstacles.setMask(bitmap);
+    //QBitmap bitmap(w,h);
+    //bitmap.clear();
+    //maps.obstacles.setMask(bitmap);
     maps.obstacles.fill(Qt::transparent);
 
     QPainter painter(&maps.obstacles);
@@ -833,9 +1101,9 @@ void Canvas::DrawRewards()
     int w = width();
     int h = height();
     maps.reward= QPixmap(w,h);
-    QBitmap bitmap(w,h);
-    bitmap.clear();
-    maps.reward.setMask(bitmap);
+    //QBitmap bitmap(w,h);
+    //bitmap.clear();
+    //maps.reward.setMask(bitmap);
     maps.reward.fill(Qt::transparent);
 
     if(!data->GetReward()->rewards) return;
@@ -948,20 +1216,20 @@ void Canvas::DrawTrajectories(QPainter &painter)
         case 0: // do nothing
             break;
         case 1: // uniform resampling
-        {
-            if(i < sequences.size()-1 || !bDrawing)
             {
-                trajectory = interpolate(trajectory, trajectoryResampleCount);
+                if(i < sequences.size()-1 || !bDrawing)
+                {
+                    trajectory = interpolate(trajectory, trajectoryResampleCount);
+                }
             }
-        }
             break;
         case 2: // spline resampling
-        {
-            if(i < sequences.size()-1 || !bDrawing)
             {
-                trajectory = interpolateSpline(trajectory, trajectoryResampleCount);
+                if(i < sequences.size()-1 || !bDrawing)
+                {
+                    trajectory = interpolateSpline(trajectory, trajectoryResampleCount);
+                }
             }
-        }
             break;
         }
         trajectories.push_back(trajectory);
@@ -1001,9 +1269,9 @@ void Canvas::DrawTrajectories()
     if(!count || (!data->GetSequences().size() && (data->GetFlag(count-1) != _TRAJ)))
     {
         maps.trajectories = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.trajectories.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.trajectories.setMask(bitmap);
         maps.trajectories.fill(Qt::transparent);
         drawnTrajectories = 0;
     }
@@ -1029,9 +1297,9 @@ void Canvas::DrawTrajectories()
     if(!drawnTrajectories || maps.trajectories.isNull())
     {
         maps.trajectories = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.trajectories.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.trajectories.setMask(bitmap);
         maps.trajectories.fill(Qt::transparent);
         drawnTrajectories = 0;
     }
@@ -1048,10 +1316,10 @@ void Canvas::DrawTrajectories()
     vector< vector<fvec> > trajectories = data->GetTrajectories(trajectoryResampleType, trajectoryResampleCount, trajectoryCenterType, 0.1, true);
     if(bDrawing)
     {
-        vector<fvec> trajectory;
+        vector<fvec> trajectory(sequences.back().second-sequences.back().first);
         for(int i=sequences.back().first; i<sequences.back().second; i++)
         {
-            trajectory.push_back(data->GetSample(i));
+            trajectory[i-sequences.back().first] = data->GetSample(i);
         }
         if(trajectory.size()) trajectories.push_back(trajectory);
     }
@@ -1127,9 +1395,9 @@ void Canvas::DrawTimeseries()
     if(!drawnTimeseries || maps.timeseries.isNull())
     {
         maps.timeseries = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.timeseries.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.timeseries.setMask(bitmap);
         maps.timeseries.fill(Qt::transparent);
         drawnTimeseries = 0;
     }
@@ -1239,6 +1507,19 @@ void Canvas::leaveEvent(QEvent *event)
     //mouseAnchor = QPoint(-1,-1);
     repaint();
 }
+void Canvas::Clear()
+{
+    maps.grid = QPixmap();
+    maps.model = QPixmap();
+    maps.confidence = QPixmap();
+    maps.info = QPixmap();
+    maps.obstacles = QPixmap();
+    maps.trajectories = QPixmap();
+    maps.samples = QPixmap();
+    ResetSamples();
+    bNewCrosshair = true;
+    repaint();
+}
 
 void Canvas::wheelEvent(QWheelEvent *event)
 {
@@ -1316,36 +1597,98 @@ QPixmap Canvas::GetScreenshot()
     bShowCrosshair = false;
     painter.setBackgroundMode(Qt::OpaqueMode);
     painter.setBackground(Qt::white);
-    PaintStandard(painter);
+    if(!canvasType) PaintStandard(painter);
+    else if(canvasType <= 5) PaintMultivariate(painter, canvasType-2); // 0: standard, 1: 3D View, so we take out 2
+    else
+    {
+        fvec params;
+        params.push_back(xIndex);
+        params.push_back(yIndex);
+        params.push_back(zIndex);
+        PaintVariable(painter, canvasType-6, params);
+    }
     bShowCrosshair = tmp;
     return screenshot;
 }
 
-bool Canvas::DeleteData( QPointF center, float radius )
+ivec Canvas::SelectSamples(QPointF center, float radius , fvec *weights)
 {
-    bool anythingDeleted = false;
+    ivec selection;
+    if(weights) (*weights).clear();
+    int closest = 0;
+    float minDist = FLT_MAX;
     FOR(i, data->GetCount())
     {
         QPointF dataPoint = toCanvasCoords(data->GetSample(i));
         QPointF point = this->mapToParent(QPoint(dataPoint.x(), dataPoint.y()));
         point -= center;
-        if(sqrt(point.x()*point.x() + point.y()*point.y()) < radius)
+        float dist = point.x()*point.x() + point.y()*point.y();
+        if(radius > 0)
         {
+            if(!weights)
+            {
+                if(sqrtf(dist) < radius) selection.push_back(i);
+            }
+            else
+            {
+                if(sqrtf(dist) < radius*1.5f)
+                {
+                    selection.push_back(i);
+                    float weight = sqrtf(dist)/radius;
+                    (*weights).push_back(weight);
+                }
+            }
+        }
+        else
+        {
+            if(dist < minDist)
+            {
+                closest = i;
+                minDist = dist;
+            }
+        }
+    }
+    if(radius < 0)
+    {
+        selection.push_back(closest);
+    }
+    return selection;
+}
+
+bool Canvas::DeleteData( QPointF center, float radius )
+{
+    bool anythingDeleted = false;
+    FOR (i, data->GetCount()) {
+        QPointF dataPoint = toCanvasCoords(data->GetSample(i));
+        QPointF point = this->mapToParent(QPoint(dataPoint.x(), dataPoint.y()));
+        point -= center;
+        if (sqrt(point.x()*point.x() + point.y()*point.y()) < radius) {
             anythingDeleted = true;
             data->RemoveSample(i);
             i--;
         }
     }
-    FOR(i, data->GetObstacles().size())
-    {
+    FOR (i, data->GetObstacles().size()) {
         QPointF obstaclePoint= toCanvasCoords(data->GetObstacle(i).center);
         QPointF point = this->mapToParent(QPoint(obstaclePoint.x(), obstaclePoint.y()));
         point -= center;
+        if (sqrt(point.x()*point.x() + point.y()*point.y()) < radius) {
+            anythingDeleted = true;
+            data->RemoveObstacle(i);
+            i--;
+        }
+    }
+    FOR (i, targets.size()) {
+        QPointF targetPoint= toCanvasCoords(targets[i]);
+        QPointF point = this->mapToParent(QPoint(targetPoint.x(), targetPoint.y()));
+        point -= center;
         if(sqrt(point.x()*point.x() + point.y()*point.y()) < radius)
         {
             anythingDeleted = true;
-            data->RemoveObstacle(i);
+            targets.erase(targets.begin() + i);
+            targetAge.erase(targetAge.begin() + i);
             i--;
+            anythingDeleted = true;
         }
     }
     return anythingDeleted;
@@ -1355,12 +1698,11 @@ void Canvas::PaintReward(fvec sample, float radius, float shift)
 {
     int w = width();
     int h = height();
-    if(maps.reward.isNull())
-    {
+    if (maps.reward.isNull()) {
         maps.reward = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.reward.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.reward.setMask(bitmap);
         maps.reward.fill(Qt::transparent);
         maps.reward.fill(Qt::white);
     }
@@ -1370,13 +1712,10 @@ void Canvas::PaintReward(fvec sample, float radius, float shift)
 
     QPointF center = toCanvasCoords(sample);
     QRadialGradient gradient( center, radius*.75);
-    if(shift > 0)
-    {
+    if (shift > 0) {
         gradient.setColorAt(0, QColor(255,0,0,shift*255));
         gradient.setColorAt(1, QColor(255,0,0,0));
-    }
-    else
-    {
+    } else {
         gradient.setColorAt(0, QColor(255,255,255,-shift*255));
         gradient.setColorAt(1, QColor(255,255,255,0));
     }
@@ -1391,12 +1730,11 @@ void Canvas::PaintGaussian(QPointF position, double variance)
 {
     int w = width();
     int h = height();
-    if(maps.reward.isNull())
-    {
+    if (maps.reward.isNull()) {
         maps.reward = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.reward.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.reward.setMask(bitmap);
         maps.reward.fill(Qt::transparent);
         maps.reward.fill(Qt::white);
     }
@@ -1410,11 +1748,9 @@ void Canvas::PaintGaussian(QPointF position, double variance)
     float value;
     float minVal = 1e30, maxVal = -1e30;
     qDebug() << "gaussian dropped at position " << position;
-    FOR(i, w)
-    {
+    FOR (i, w) {
         point.x = i/(float)w;
-        FOR(j, h)
-        {
+        FOR (j, h) {
             point.y = j/(float)h;
             value = (pos - point).lengthSquared();
             value = expf(-0.5*value*invSigma);
@@ -1441,9 +1777,9 @@ void Canvas::PaintGradient(QPointF position)
     if(maps.reward.isNull())
     {
         maps.reward = QPixmap(w,h);
-        QBitmap bitmap(w,h);
-        bitmap.clear();
-        maps.reward.setMask(bitmap);
+        //QBitmap bitmap(w,h);
+        //bitmap.clear();
+        //maps.reward.setMask(bitmap);
         maps.reward.fill(Qt::transparent);
         maps.reward.fill(Qt::white);
     }
@@ -1460,3 +1796,63 @@ void Canvas::PaintGradient(QPointF position)
     painter.setPen(Qt::NoPen);
     painter.drawRect(maps.reward.rect());
 }
+
+QString Canvas::GetClassString(int classNumber)
+{
+    QString className = QString("Class %1").arg(classNumber);
+    if (classNames.count(classNumber)) {
+        QString name = classNames[classNumber];
+        if (name.length() < 3) name = "Class " + name;
+        return name;
+    }
+    return className;
+}
+
+QRgb Canvas::GetColorMapValue(float value, int colorscheme=2)
+{
+    float r, g, b;
+    switch(colorscheme)
+    {
+    case 0:
+        {
+            r = value;
+            g = 0;
+            b = 0;
+        }
+        break;
+    case 1: // autumn
+        {
+            r = value;
+            g = value*0.6;
+            b = value*0.2;
+        }
+        break;
+    case 2: // jet
+        {
+            float Red = 0, Green = 0, Blue = 0;
+
+            if (value < 0.5f) Red = value * 2;
+            else Red = (1.0f - value) * 2;
+
+            if (value >= 0.3f && value < 0.8f) Green = (value - 0.3f) * 2;
+            else if (value < 0.3f) Green = (0.3f - value) * 2;
+            else Green = (1.3f - value) * 2;
+
+            if (value >= 0.5f) Blue = (value - 0.5f) * 2;
+            else Blue = (0.5f - value) * 2;
+
+            r = Red;
+            g = Green;
+            b = Blue;
+        }
+        break;
+    case 3: // grayscale
+        {
+            r = value;
+            g = value;
+            b = value;
+        }
+        break;
+    }
+    return qRgb(r*255,g*255,b*255);
+}
