@@ -34,7 +34,7 @@ using namespace std;
 /******************************************/
 u32 DatasetManager::IDCount = 0;
 
-DatasetManager::DatasetManager(int dimension)
+DatasetManager::DatasetManager(const int dimension)
 : size(dimension)
 {
     bProjected = false;
@@ -56,14 +56,22 @@ void DatasetManager::Clear()
 	labels.clear();
 	sequences.clear();
 	rewards.Clear();
+    categorical.clear();
 	KILL(perm);
 }
 
-void DatasetManager::AddSample(fvec sample, int label, dsmFlags flag)
+void DatasetManager::AddSample(const fvec sample, const int label, const dsmFlags flag)
 {
 	if (!sample.size()) return;
+    int dim = GetDimCount();
 	size = sample.size();
-
+    if(dim != size) // we need to go through all our data and adjust the dimensions
+    {
+        FOR(i, samples.size())
+        {
+            while(samples[i].size() < size) samples[i].push_back(0.f);
+        }
+    }
 	samples.push_back(sample);
 	labels.push_back(label);
 	flags.push_back(flag);
@@ -71,11 +79,20 @@ void DatasetManager::AddSample(fvec sample, int label, dsmFlags flag)
 	perm = randPerm(samples.size());
 }
 
-void DatasetManager::AddSamples(std::vector< fvec > newSamples, ivec newLabels, std::vector<dsmFlags> newFlags)
+void DatasetManager::AddSamples(const std::vector< fvec > newSamples, const ivec newLabels, const std::vector<dsmFlags> newFlags)
 {
-	FOR(i, newSamples.size())
+    if(!newSamples.size()) return;
+    int dim = GetDimCount();
+    size = newSamples[0].size();
+    if(dim != size) // we need to go through all our data and adjust the dimensions
+    {
+        FOR(i, samples.size())
+        {
+            while(samples[i].size() < size) samples[i].push_back(0.f);
+        }
+    }
+    FOR(i, newSamples.size())
 	{
-		if(newSamples[0].size()) size = newSamples[0].size();
 		if(newSamples[i].size())
 		{
 			samples.push_back(newSamples[i]);
@@ -89,12 +106,12 @@ void DatasetManager::AddSamples(std::vector< fvec > newSamples, ivec newLabels,
 	perm = randPerm(samples.size());
 }
 
-void DatasetManager::AddSamples(DatasetManager &newSamples)
+void DatasetManager::AddSamples(const DatasetManager &newSamples)
 {
 	AddSamples(newSamples.GetSamples(), newSamples.GetLabels(), newSamples.GetFlags());
 }
 
-void DatasetManager::RemoveSample(unsigned int index)
+void DatasetManager::RemoveSample(const unsigned int index)
 {
 	if(index >= samples.size()) return;
 	if(samples.size() == 1)
@@ -156,7 +173,7 @@ void DatasetManager::RemoveSamples(ivec indices)
     }
 }
 
-void DatasetManager::AddSequence(int start, int stop)
+void DatasetManager::AddSequence(const int start, const int stop)
 {
 	if(start >= samples.size() || stop >= samples.size()) return;
 	for(int i=start; i<=stop; i++) flags[i] = _TRAJ;
@@ -165,7 +182,7 @@ void DatasetManager::AddSequence(int start, int stop)
 	std::sort(sequences.begin(), sequences.end());
 }
 
-void DatasetManager::AddSequence(ipair newSequence)
+void DatasetManager::AddSequence(const ipair newSequence)
 {
 	if(newSequence.first >= samples.size() || newSequence.second >= samples.size()) return;
 	for(int i=newSequence.first; i<=newSequence.second; i++) flags[i] = _TRAJ;
@@ -174,7 +191,7 @@ void DatasetManager::AddSequence(ipair newSequence)
 	std::sort(sequences.begin(), sequences.end());
 }
 
-void DatasetManager::AddSequences(std::vector< ipair > newSequences)
+void DatasetManager::AddSequences(const std::vector< ipair > newSequences)
 {
 	sequences.reserve(sequences.size()+newSequences.size());
 	FOR(i, newSequences.size())
@@ -183,14 +200,14 @@ void DatasetManager::AddSequences(std::vector< ipair > newSequences)
 	}
 }
 
-void DatasetManager::RemoveSequence(unsigned int index)
+void DatasetManager::RemoveSequence(const unsigned int index)
 {
 	if(index >= sequences.size()) return;
 	for(int i=index; i<sequences.size()-1; i++) sequences[i] = sequences[i+1];
 	sequences.pop_back();
 }
 
-void DatasetManager::AddTimeSerie(std::string name, std::vector<fvec> data, std::vector<long int>  timestamps)
+void DatasetManager::AddTimeSerie(const std::string name, const std::vector<fvec> data, const std::vector<long int>  timestamps)
 {
 	TimeSerie serie;
 	serie.name = name;
@@ -199,23 +216,23 @@ void DatasetManager::AddTimeSerie(std::string name, std::vector<fvec> data, std:
 	AddTimeSerie(serie);
 }
 
-void DatasetManager::AddTimeSerie(TimeSerie serie)
+void DatasetManager::AddTimeSerie(const TimeSerie serie)
 {
 	series.push_back(serie);
 }
 
-void DatasetManager::AddTimeSeries(std::vector< TimeSerie > newTimeSeries)
+void DatasetManager::AddTimeSeries(const vector<TimeSerie> newTimeSeries)
 {
 	series.insert(series.end(), newTimeSeries.begin(), newTimeSeries.end());
 }
 
-void DatasetManager::RemoveTimeSerie(unsigned int index)
+void DatasetManager::RemoveTimeSerie(const unsigned int index)
 {
 	if(index >= series.size()) return;
 	series.erase(series.begin() + index);
 }
 
-void DatasetManager::AddObstacle(fvec center, fvec axes, float angle, fvec power, fvec repulsion)
+void DatasetManager::AddObstacle(const fvec center, const fvec axes, const float angle, const fvec power, const fvec repulsion)
 {
 	Obstacle o;
 	o.center = center;
@@ -226,26 +243,26 @@ void DatasetManager::AddObstacle(fvec center, fvec axes, float angle, fvec power
 	obstacles.push_back(o);
 }
 
-void DatasetManager::AddObstacles(std::vector<Obstacle> newObstacles)
+void DatasetManager::AddObstacles(const std::vector<Obstacle> newObstacles)
 {
 	FOR(i, newObstacles.size()) obstacles.push_back(newObstacles[i]);
 }
 
-void DatasetManager::RemoveObstacle(unsigned int index)
+void DatasetManager::RemoveObstacle(const unsigned int index)
 {
 	if(index >= obstacles.size()) return;
 	for(int i=index; i<obstacles.size()-1; i++) obstacles[i] = obstacles[i+1];
 	obstacles.pop_back();
 }
 
-void DatasetManager::AddReward(float *values, ivec size, fvec lowerBoundary, fvec higherBoundary)
+void DatasetManager::AddReward(const float *values, const ivec size, const fvec lowerBoundary, const fvec higherBoundary)
 {
 	rewards.SetReward(values, size, lowerBoundary, higherBoundary);
 }
 
 // we compare the current sample with all the ones in the dataset
 // and return the smallest distance
-double DatasetManager::Compare(fvec sample)
+double DatasetManager::Compare(const fvec sample) const
 {
 	if(!sample.size()) return 1.0;
 
@@ -266,7 +283,7 @@ double DatasetManager::Compare(fvec sample)
 	return minDist;
 }
 
-void DatasetManager::Randomize(int seed)
+void DatasetManager::Randomize(const int seed)
 {
 	KILL(perm);
 	if(samples.size()) perm = randPerm(samples.size(), seed);
@@ -277,15 +294,57 @@ void DatasetManager::ResetFlags()
 	FOR(i, samples.size()) flags[i] = _UNUSED;
 }
 
-void DatasetManager::SetSample(int index, fvec sample)
+void DatasetManager::SetSample(const int index, const fvec sample)
 {
-	if(index >= 0 && index < samples.size()) samples[index] = sample;
+    if(index >= 0 && index < samples.size()) samples[index] = sample;
 }
 
-fvec DatasetManager::GetSampleDim(int index, ivec inputDims, int outputDim)
+string DatasetManager::GetCategorical(const int dimension, const int value) const
 {
-    if(index>=samples.size()) return fvec();
+    string s;
+    if(categorical.count(dimension) && value < categorical.at(dimension).size())
+    {
+        s = categorical.at(dimension)[value];
+    }
+    return s;
+}
+
+bool DatasetManager::IsCategorical(const int dimension) const
+{
+    return categorical.count(dimension) > 0;
+}
+
+fvec DatasetManager::GetSampleDim(const int index, const ivec inputDims, const int outputDim) const
+{
+    if(index >= samples.size()) return fvec();
     if(!inputDims.size()) return samples[index];
+    if(outputDim == -1)
+    {
+        fvec sample(inputDims.size());
+        FOR(d, inputDims.size()) sample[d] = samples[index][inputDims[d]];
+        return sample;
+    }
+    else
+    {
+        int outputIndex = -1;
+        FOR(d, inputDims.size())
+        {
+            if(outputDim == inputDims[d])
+            {
+                outputIndex = d;
+                break;
+            }
+        }
+        fvec sample(inputDims.size() + (outputIndex==-1 ? 0 : 1));
+        FOR(d, inputDims.size())
+        {
+            if(d==outputIndex) sample.back() = samples[index][inputDims[d]];
+            else sample[d<outputIndex ? d : d-1] = samples[index][inputDims[d]];
+        }
+        if(outputIndex == -1) sample.back() = samples[index][outputDim];
+        return sample;
+    }
+
     int dim = inputDims.size();
     fvec sample(dim + outputDim!=-1?1:0);
     FOR(d, dim) sample[d] = samples[index][inputDims[d]];
@@ -293,26 +352,89 @@ fvec DatasetManager::GetSampleDim(int index, ivec inputDims, int outputDim)
     return sample;
 }
 
-std::vector< fvec > DatasetManager::GetSampleDims(ivec inputDims, int outputDim)
+std::vector< fvec > DatasetManager::GetSampleDims(const ivec inputDims, const int outputDim) const
 {
-    if(!inputDims.size()) return samples;
+    if ( !inputDims.size() ) return samples;
 
     vector<fvec> newSamples = samples;
-    int newDim = inputDims.size() + (outputDim != -1 ? 1 : 0);
-    FOR(i, samples.size())
-    {
-        fvec newSample(newDim);
-        FOR(d, inputDims.size())
-        {
-            newSample[d] = samples[i][inputDims[d]];
+    int newDim = inputDims.size();
+    if ( outputDim == -1 ) {
+        FOR ( i, samples.size() ) {
+            fvec newSample(newDim);
+            FOR ( d, inputDims.size() ) {
+                newSample[d] = samples[i][inputDims[d]];
+            }
+            newSamples[i] = newSample;
+        }
+    } else {
+        int outputIndex = -1;
+        FOR ( d, inputDims.size() ) {
+            if ( outputDim == inputDims[d] ) {
+                newDim--;
+                break;
+            }
+        }
+        FOR ( i, samples.size() ) {
+            fvec newSample(newDim);
+            if ( outputIndex == -1 ) {
+                FOR ( d, newDim-1 ) {
+                    newSample[d] = samples[i][inputDims[d]];
+                }
+                newSample[newDim-1] = samples[i][outputDim];
+            } else {
+                FOR ( d, newDim ) {
+                    if ( d == outputIndex ) newSample[newDim-1] = samples[i][inputDims[d]];
+                    else newSample[d<outputIndex?d:d-1] = samples[i][inputDims[d]];
+                }
+            }
+            newSamples[i] = newSample;
         }
-        if(outputDim != -1) newSample[newDim-1] = samples[i][outputDim];
-        newSamples[i] = newSample;
     }
     return newSamples;
 }
 
-std::vector< fvec > DatasetManager::GetSamples(u32 count, dsmFlags flag, dsmFlags replaceWith)
+std::vector< fvec > DatasetManager::GetSampleDims(const std::vector<fvec> samples, const ivec inputDims, const int outputDim) const
+{
+    if(!inputDims.size()) return samples;
+
+    vector<fvec> newSamples = samples;
+    int newDim = inputDims.size();
+    if ( outputDim == -1 ) {
+        FOR ( i, samples.size() ) {
+            fvec newSample(newDim);
+            FOR ( d, inputDims.size() ) {
+                newSample[d] = samples[i][inputDims[d]];
+            }
+            newSamples[i] = newSample;
+        }
+    } else {
+        int outputIndex = -1;
+        FOR ( d, inputDims.size() ) {
+            if ( outputDim == inputDims[d] ) {
+                newDim--;
+                break;
+            }
+        }
+        FOR ( i, samples.size() ) {
+            fvec newSample(newDim);
+            if ( outputIndex == -1 ) {
+                FOR ( d, newDim-1 ) {
+                    newSample[d] = samples[i][inputDims[d]];
+                }
+                newSample[newDim-1] = samples[i][outputDim];
+            } else {
+                FOR ( d, newDim ) {
+                    if ( d == outputIndex ) newSample[newDim-1] = samples[i][inputDims[d]];
+                    else newSample[d<outputIndex?d:d-1] = samples[i][inputDims[d]];
+                }
+            }
+            newSamples[i] = newSample;
+        }
+    }
+    return newSamples;
+}
+
+std::vector< fvec > DatasetManager::GetSamples(const u32 count, const dsmFlags flag, const dsmFlags replaceWith)
 {
 	std::vector< fvec > selected;
 	if (!samples.size() || !perm) return selected;
@@ -343,9 +465,9 @@ std::vector< fvec > DatasetManager::GetSamples(u32 count, dsmFlags flag, dsmFlag
 	return selected;
 }
 
-std::vector< std::vector < fvec > > DatasetManager::GetTrajectories(int resampleType, int resampleCount, int centerType, float dT, int zeroEnding)
+std::vector< std::vector < fvec > > DatasetManager::GetTrajectories(const int resampleType, const int resampleCount_, const int centerType, const float dT, const int zeroEnding) const
 {
-
+    int resampleCount = resampleCount_;
 	// we split the data into trajectories
 	vector< vector<fvec> > trajectories;
 	if(!sequences.size() || !samples.size()) return trajectories;
@@ -501,8 +623,9 @@ std::vector< std::vector < fvec > > DatasetManager::GetTrajectories(int resample
 
 void DatasetManager::Save(const char *filename)
 {
-	if(!samples.size()) return;
+    if(!samples.size() && rewards.Empty()) return;
 	u32 sampleCnt = samples.size();
+    if(sampleCnt) size = samples[0].size();
 
 	ofstream file(filename);
 	if(!file.is_open()) return;
@@ -529,7 +652,7 @@ void DatasetManager::Save(const char *filename)
 	}
 
 	// we load the obstacles
-	if(obstacles.size())
+    if(obstacles.size())
 	{
 		file << "o " << obstacles.size() << "\n";
 		FOR(i, obstacles.size())
@@ -543,6 +666,18 @@ void DatasetManager::Save(const char *filename)
 			file << obstacles[i].repulsion[1] << "\n";
 		}
 	}
+    if(!rewards.Empty())
+    {
+        file << "r " << rewards.dim << " " << rewards.length << "\n";
+        FOR(i, rewards.dim)
+        {
+            file << rewards.size[i] << " " << rewards.lowerBoundary[i] << " " << rewards.higherBoundary[i] << "\n";
+        }
+        FOR(i, rewards.length)
+        {
+            file << rewards.rewards[i] << " ";
+        }
+    }
 
 	file.close();
 }
@@ -616,6 +751,39 @@ bool DatasetManager::Load(const char *filename)
 			obstacles.push_back(obstacle);
 		}
 	}
+    // we load the reward
+    if(nextChar == 'r')
+    {
+        char dump;
+        file >> dump;
+        int dims, length;
+        file >> dims >> length;
+        ivec size(dims);
+        fvec lowerBoundary(dims), higherBoundary(dims);
+        int testLength = 1;
+        FOR(i, dims)
+        {
+            file >> size[i] >> lowerBoundary[i] >> higherBoundary[i];
+            testLength *= size[i];
+        }
+        if(testLength == length)
+        {
+            double *rewardData = new double[length];
+            FOR(i, length)
+            {
+                double value;
+                file >> value;
+                rewardData[i] = value;
+            }
+            rewards.lowerBoundary = lowerBoundary;
+            rewards.higherBoundary = higherBoundary;
+            rewards.size = size;
+            rewards.dim = dims;
+            rewards.length = length;
+            if(rewards.rewards) delete [] rewards.rewards;
+            rewards.rewards = rewardData;
+        }
+    }
 
 	file.close();
 	KILL(perm);
@@ -623,46 +791,44 @@ bool DatasetManager::Load(const char *filename)
 	return samples.size() > 0;
 }
 
-int DatasetManager::GetDimCount()
+int DatasetManager::GetDimCount() const
 {
 	int dim = 2;
-	if(samples.size()) dim = samples[0].size();
-	if(series.size() && series[0].size())
+    if(samples.size()) dim = samples.at(0).size();
+    if(series.size() && series.at(0).size())
 	{
-		dim = series[0][0].size()+1;
+        dim = series.at(0).at(0).size()+1;
 	}
 	return dim;
 }
 
-std::pair<fvec, fvec> DatasetManager::GetBounds()
+std::pair<fvec, fvec> DatasetManager::GetBounds() const
 {
     if(!samples.size()) return make_pair(fvec(),fvec());
     int dim = samples[0].size();
     fvec mins(dim,FLT_MAX), maxes(dim,-FLT_MAX);
     FOR(i, samples.size())
     {
-        fvec& sample = samples[i];
+        const fvec& sample = samples.at(i);
         int dim = sample.size();
         FOR(d,dim)
         {
-            if(mins[d] > sample[d]) mins[d] = sample[d];
-            if(maxes[d] < sample[d]) maxes[d] = sample[d];
+            if(mins[d] > sample[d]) mins[d] = sample.at(d);
+            if(maxes[d] < sample[d]) maxes[d] = sample.at(d);
         }
     }
     return make_pair(mins, maxes);
 }
 
-u32 DatasetManager::GetClassCount(ivec classes)
+u32 DatasetManager::GetClassCount(const ivec classes)
 {
-	u32 counts[256];
-	memset(counts, 0, 256*sizeof(u32));
-	FOR(i, classes.size()) counts[classes[i]]++;
-	u32 result = 0;
-	for (u32 i=1; i<256; i++) result += counts[i] > 0 ? 1 : 0;
-	return result;
+    int cnt=0;
+    map<int,int> classMap;
+    FOR(i, classes.size()) if(!classMap.count(classes[i])) classMap[classes[i]] = cnt++;
+    return classMap.size();
 }
 
-bvec DatasetManager::GetFreeFlags()
+bvec DatasetManager::GetFreeFlags() const
 {
 	bvec res;
 	FOR(i, flags.size()) res.push_back(flags[i] == _UNUSED);
@@ -680,17 +846,20 @@ RewardMap& RewardMap::operator= (const RewardMap& r)
   if (this != &r) {
 	  dim = r.dim;
 	  size = r.size;
-	  length = r.length;
 	  lowerBoundary = r.lowerBoundary;
 	  higherBoundary = r.higherBoundary;
-	  if(rewards) delete [] rewards;
-	  rewards = new float[length];
-	  memcpy(rewards, r.rewards, length*sizeof(float));
+      if(length != r.length)
+      {
+          length = r.length;
+          if(rewards) delete [] rewards;
+          rewards = new double[length];
+      }
+      memcpy(rewards, r.rewards, length*sizeof(double));
   }
   return *this;
 }
 
-void RewardMap::SetReward(float *rewards, ivec size, fvec lowerBoundary, fvec higherBoundary)
+void RewardMap::SetReward(const double *rewards, const ivec size, const fvec lowerBoundary, const fvec higherBoundary)
 {
 	this->lowerBoundary = lowerBoundary;
 	this->higherBoundary = higherBoundary;
@@ -699,8 +868,32 @@ void RewardMap::SetReward(float *rewards, ivec size, fvec lowerBoundary, fvec hi
 	length = 1;
 	FOR(i, size.size()) length *= size[i];
 	if(this->rewards) delete [] this->rewards;
-	this->rewards = new float[length];
-	memcpy(this->rewards, rewards, length*sizeof(float));
+    this->rewards = new double[length];
+    memcpy(this->rewards, rewards, length*sizeof(double));
+}
+
+void RewardMap::SetReward(const float *rewards, const ivec size, const fvec lowerBoundary, const fvec higherBoundary)
+{
+    this->lowerBoundary = lowerBoundary;
+    this->higherBoundary = higherBoundary;
+    this->size = size;
+    dim = size.size();
+    length = 1;
+    FOR(i, size.size()) length *= size[i];
+    if(this->rewards) delete [] this->rewards;
+    this->rewards = new double[length];
+    FOR(i, length)
+    {
+        this->rewards[i] = (double)rewards[i];
+    }
+}
+
+float *RewardMap::GetRewardFloat() const
+{
+    if(!length) return 0;
+    float *rewards = new float[length];
+    FOR(i, length) rewards[i] = this->rewards[i];
+    return rewards;
 }
 
 void RewardMap::Clear()
@@ -710,7 +903,7 @@ void RewardMap::Clear()
 	length = 0;
 	lowerBoundary.clear();
 	higherBoundary.clear();
-	if(rewards) delete [] rewards;
+    KILL(rewards);
 }
 
 void RewardMap::Zero()
@@ -719,7 +912,7 @@ void RewardMap::Zero()
 }
 
 // return the value of the reward function at the coordinates provided
-float RewardMap::ValueAt(fvec sample)
+float RewardMap::ValueAt(fvec sample) const
 {
 	if(!rewards) return 0.f;
 	ivec index;
@@ -745,7 +938,7 @@ float RewardMap::ValueAt(fvec sample)
 	return rewards[rewardIndex];
 }
 
-void RewardMap::SetValueAt(fvec sample, float value)
+void RewardMap::SetValueAt(const fvec sample, const double value)
 {
 	if(!rewards) return;
 	ivec index;
@@ -768,7 +961,7 @@ void RewardMap::SetValueAt(fvec sample, float value)
 	rewards[rewardIndex] = value;
 }
 
-void RewardMap::ShiftValueAt(fvec sample, float shift)
+void RewardMap::ShiftValueAt(const fvec sample, const double shift)
 {
 	if(!rewards) return;
 	ivec index;
@@ -791,7 +984,7 @@ void RewardMap::ShiftValueAt(fvec sample, float shift)
 	rewards[rewardIndex] += shift;
 }
 
-void RewardMap::ShiftValueAt(fvec sample, float radius, float shift)
+void RewardMap::ShiftValueAt(const fvec sample, const double radius, const double shift)
 {
 	if(!rewards) return;
 	ivec index;
