@@ -1,4 +1,5 @@
 #include <public.h>
+#include <basicMath.h>
 #include <mymaths.h>
 #include <basicMath.h>
 #include "expose.h"
@@ -6,6 +7,7 @@
 #include <QClipboard>
 #include <QBitmap>
 #include <QDebug>
+#include <QProgressDialog>
 
 using namespace std;
 
@@ -81,7 +83,11 @@ void Expose::DrawData(QPixmap& pixmap, std::vector<fvec> samples, std::vector<QC
         mapW = w/gridX - pad*2;
         mapH = h/gridX - pad*2;
         int radiusBase = max(5.f, 5 * sqrtf(mapW / 200.f));
-
+#ifndef WIN32
+        QProgressDialog progress("Generating Scatterplots...", "Abort", 0, dim*dim, NULL);
+        progress.setWindowModality(Qt::WindowModal);
+        int progressCnt = 0;
+#endif
         QList<QPixmap> maps;
         FOR(index0, dim)
         {
@@ -92,9 +98,9 @@ void Expose::DrawData(QPixmap& pixmap, std::vector<fvec> samples, std::vector<QC
                 if(!bLearned) map.fill(Qt::white);
                 else
                 {
-                    QBitmap bitmap(map.size());
-                    bitmap.clear();
-                    map.setMask(bitmap);
+                    //QBitmap bitmap(map.size());
+                    //bitmap.clear();
+                    //map.setMask(bitmap);
                     map.fill(Qt::transparent);
                 }
                 QPainter painter(&map);
@@ -138,8 +144,18 @@ void Expose::DrawData(QPixmap& pixmap, std::vector<fvec> samples, std::vector<QC
                 if(bProjected) text = QString("e%1  e%2").arg(index1+1).arg(index0+1);
                 painter.drawText(pad/2+1, map.height()-pad/2-1,text);
                 maps.push_back(map);
+#ifndef WIN32
+                progress.setValue(progressCnt++);
+                progress.update();
+                if (progress.wasCanceled()) break;
+            }
+            if (progress.wasCanceled()) break;
+        }
+        progress.setValue(dim*dim);
+#else
             }
         }
+#endif
 
         FOR(i, maps.size())
         {
@@ -373,9 +389,9 @@ void Expose::DrawTrajectories(QPixmap& pixmap, vector< vector<fvec> > trajectori
                 QPixmap map(mapW + 2*pad,mapH + 2*pad);
                 int smallW = map.width() - 2*pad, smallH = map.height() - 2*pad;
 
-                QBitmap bitmap(map.size());
-                bitmap.clear();
-                map.setMask(bitmap);
+                //QBitmap bitmap(map.size());
+                //bitmap.clear();
+                //map.setMask(bitmap);
                 map.fill(Qt::transparent);
 
                 QPainter painter(&map);
