@@ -19,16 +19,14 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *********************************************************************/
 #include "mldemos.h"
 #include <QtGui/QApplication>
-#include <QtPlugin>
-
-//Q_IMPORT_PLUGIN(mld_classifySVM)
 
 int main(int argc, char *argv[])
 {
     QApplication a(argc, argv);
-	QString filename = "";
-	if(argc > 1) filename = QString(argv[1]);
-	MLDemos w(filename);
+
+    QString filename = "";
+    if(argc > 1) filename = QString(argv[1]);
+    MLDemos w(filename);
     w.show();
     return a.exec();
 }
