@@ -3,7 +3,7 @@
 #ifndef DLIB_CMD_LINE_PARSER_CHECk_1_
 #define DLIB_CMD_LINE_PARSER_CHECk_1_ 
 
-#include "cmd_line_parser_kernel_abstract.h"
+#include "cmd_line_parser_check_abstract.h"
 #include <sstream>
 #include <string>
 #include "../string.h"
@@ -191,6 +191,11 @@ namespace dlib
             const string_type& option_name2
         ) const;
 
+        void check_sub_option (
+            const string_type& parent_option,
+            const string_type& sub_option
+        ) const;
+
         template <
             size_t length
             >
@@ -240,7 +245,7 @@ namespace dlib
     {
         try
         {
-            const typename clp_base::option_type& opt = option(option_name);
+            const typename clp_base::option_type& opt = this->option(option_name);
             const unsigned long number_of_arguments = opt.number_of_arguments();
             const unsigned long count = opt.count();
             for (unsigned long i = 0; i < number_of_arguments; ++i)
@@ -270,7 +275,7 @@ namespace dlib
     {
         try
         {
-            const typename clp_base::option_type& opt = option(option_name);
+            const typename clp_base::option_type& opt = this->option(option_name);
             const unsigned long number_of_arguments = opt.number_of_arguments();
             const unsigned long count = opt.count();
             for (unsigned long i = 0; i < number_of_arguments; ++i)
@@ -307,7 +312,7 @@ namespace dlib
     {
         try
         {
-            const typename clp_base::option_type& opt = option(option_name);
+            const typename clp_base::option_type& opt = this->option(option_name);
             const unsigned long number_of_arguments = opt.number_of_arguments();
             const unsigned long count = opt.count();
             for (unsigned long i = 0; i < number_of_arguments; ++i)
@@ -348,7 +353,7 @@ namespace dlib
         const char_type* (&arg_set)[length]
     ) const
     {
-        const typename clp_base::option_type& opt = option(option_name);
+        const typename clp_base::option_type& opt = this->option(option_name);
         const unsigned long number_of_arguments = opt.number_of_arguments();
         const unsigned long count = opt.count();
         for (unsigned long i = 0; i < number_of_arguments; ++i)
@@ -386,8 +391,8 @@ namespace dlib
         {
             for (size_t j = i+1; j < length; ++j)
             {
-                if (option(option_set[i]).count() > 0 &&
-                    option(option_set[j]).count() > 0 )
+                if (this->option(option_set[i]).count() > 0 &&
+                    this->option(option_set[j]).count() > 0 )
                 {
                     throw cmd_line_check_error(
                         EINCOMPATIBLE_OPTIONS,
@@ -410,8 +415,8 @@ namespace dlib
         const string_type& option_name2
     ) const
     {
-        if (option(option_name1).count() > 0 &&
-            option(option_name2).count() > 0 )
+        if (this->option(option_name1).count() > 0 &&
+            this->option(option_name2).count() > 0 )
         {
             throw cmd_line_check_error(
                 EINCOMPATIBLE_OPTIONS,
@@ -423,6 +428,27 @@ namespace dlib
         }
     }
 
+// ----------------------------------------------------------------------------------------
+
+    template <typename clp_base>
+    void cmd_line_parser_check_1<clp_base>::
+    check_sub_option (
+        const string_type& parent_option,
+        const string_type& sub_option
+    ) const
+    {
+        if (this->option(parent_option).count() == 0)
+        {
+            if (this->option(sub_option).count() != 0)
+            {
+                std::vector<string_type> vect;
+                vect.resize(1);
+                vect[0] = parent_option;
+                throw cmd_line_check_error( EMISSING_REQUIRED_OPTION, sub_option, vect);
+            }
+        }
+    }
+
 // ----------------------------------------------------------------------------------------
 
     template <typename clp_base>
@@ -433,12 +459,12 @@ namespace dlib
         const char_type* (&sub_option_set)[length]
     ) const
     {
-        if (option(parent_option).count() == 0)
+        if (this->option(parent_option).count() == 0)
         {
             size_t i = 0;
             for (; i < length; ++i)
             {
-                if (option(sub_option_set[i]).count() > 0)
+                if (this->option(sub_option_set[i]).count() > 0)
                     break;
             }
             if (i != length)
@@ -462,13 +488,13 @@ namespace dlib
     ) const
     {
         // first check if the sub_option is present
-        if (option(sub_option).count() > 0)
+        if (this->option(sub_option).count() > 0)
         {
             // now check if any of the parents are present
             bool parents_present = false;
             for (size_t i = 0; i < length; ++i)
             {
-                if (option(parent_option_set[i]).count() > 0)
+                if (this->option(parent_option_set[i]).count() > 0)
                 {
                     parents_present = true;
                     break;
@@ -497,7 +523,7 @@ namespace dlib
         bool parents_present = false;
         for (size_t i = 0; i < parent_length; ++i)
         {
-            if (option(parent_option_set[i]).count() > 0)
+            if (this->option(parent_option_set[i]).count() > 0)
             {
                 parents_present = true;
                 break;
@@ -510,7 +536,7 @@ namespace dlib
             size_t i = 0;
             for (; i < sub_length; ++i)
             {
-                if (option(sub_option_set[i]).count() > 0)
+                if (this->option(sub_option_set[i]).count() > 0)
                     break;
             }
             if (i != sub_length)
@@ -533,7 +559,7 @@ namespace dlib
         size_t i = 0;
         for (; i < length; ++i)
         {
-            if (option(option_set[i]).count() > 1)
+            if (this->option(option_set[i]).count() > 1)
                 break;
         }
         if (i != length)
