@@ -90,9 +90,10 @@ namespace dlib
                 - #bottom() == p.y()
         !*/
 
+        template <typename T>
         rectangle (
-            const point& p1,
-            const point& p2
+            const vector<T,2>& p1,
+            const vector<T,2>& p2
         );
         /*!
             ensures
@@ -401,6 +402,26 @@ namespace dlib
               The data in the input stream should be of the form [(left, top) (right, bottom)]
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    point center (
+        const dlib::rectangle& rect
+    );
+    /*!
+        ensures
+            - returns the center of the given rectangle
+    !*/
+
+// ----------------------------------------------------------------------------------------
+
+    dlib::vector<double,2> dcenter (
+        const dlib::rectangle& rect
+    );
+    /*!
+        ensures
+            - returns the center of the given rectangle using a real valued vector.  
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     inline const rectangle centered_rect (
@@ -411,6 +432,7 @@ namespace dlib
     /*!
         ensures
             - returns a rectangle R such that:
+                - center(R) == p
                 - if (width == 0 || height == 0)
                     - R.width() == 0 
                     - R.height() == 0 
@@ -418,7 +440,6 @@ namespace dlib
                     - R.width() == width
                     - R.height() == height 
                 - R.tl_corner() == point(p.x()-width/2, p.y()-height/2)
-                - The center of R is a the point p 
     !*/
 
 // ----------------------------------------------------------------------------------------
@@ -432,6 +453,7 @@ namespace dlib
     /*!
         ensures
             - returns a rectangle R such that:
+                - center(R) == p
                 - if (width == 0 || height == 0)
                     - R.width() == 0 
                     - R.height() == 0 
@@ -439,7 +461,6 @@ namespace dlib
                     - R.width() == width
                     - R.height() == height 
                 - R.tl_corner() == point(x-width/2, y-height/2)
-                - The center of R is a the point (x,y)
     !*/
 
 // ----------------------------------------------------------------------------------------
@@ -480,6 +501,33 @@ namespace dlib
               (i.e. grows the given rectangle by expanding its border by num)
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    inline const rectangle shrink_rect (
+        const rectangle& rect,
+        long width,
+        long height
+    );
+    /*!
+        ensures
+            - returns rectangle(rect.left()+width, rect.top()+height, rect.right()-width, rect.bottom()-height)
+              (i.e. shrinks the given rectangle by shrinking its left and right borders by width
+              and its top and bottom borders by height. )
+    !*/
+
+// ----------------------------------------------------------------------------------------
+
+    inline const rectangle grow_rect (
+        const rectangle& rect,
+        long width,
+        long height
+    );
+    /*!
+        ensures
+            - return shrink_rect(rect, -width, -height)
+              (i.e. grows the given rectangle by expanding its border)
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     const rectangle translate_rect (
@@ -606,6 +654,17 @@ namespace dlib
                 - returns the point in rect that is closest to p
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    inline long distance_to_rect_edge (
+        const rectangle& rect,
+        const point& p
+    );
+    /*!
+        ensures
+            - returns the Manhattan distance between the edge of rect and p.
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
     template <
@@ -624,6 +683,30 @@ namespace dlib
               bounding rectangle for that gridded object.)
     !*/
 
+// ----------------------------------------------------------------------------------------
+
+    inline rectangle operator+ (
+        const rectangle& r,
+        const point& p
+    );
+    /*!
+        ensures
+            - returns r + rectangle(p)
+              (i.e. returns the rectangle that contains both r and p)
+    !*/
+
+// ----------------------------------------------------------------------------------------
+
+    inline rectangle operator+ (
+        const point& p,
+        const rectangle& r
+    );
+    /*!
+        ensures
+            - returns r + rectangle(p)
+              (i.e. returns the rectangle that contains both r and p)
+    !*/
+
 // ----------------------------------------------------------------------------------------
 
 }
