@@ -5,7 +5,7 @@
 
 #include "matrix_exp_abstract.h"
 #include "../serialize.h"
-#include "../memory_manager.h"
+#include "../algs.h"
 #include "matrix_data_layout_abstract.h"
 
 namespace dlib
@@ -108,6 +108,64 @@ namespace dlib
               scalar value.  The resulting matrix will have the same dimensions as m.
     !*/
 
+    template <typename T>
+    const matrix_exp operator/ (
+        const T& value,
+        const matrix_exp& m
+    );
+    /*!
+        ensures
+            - returns the result of dividing the given scalar value by all the elements 
+              of matrix m.  The resulting matrix will have the same dimensions as m.
+    !*/
+
+    template <typename T>
+    const matrix_exp operator+ (
+        const matrix_exp& m,
+        const T& value
+    );
+    /*!
+        ensures
+            - returns the result of adding value to all the elements of matrix m.  
+              The resulting matrix will have the same dimensions as m.
+    !*/
+
+    template <typename T>
+    const matrix_exp operator+ (
+        const T& value,
+        const matrix_exp& m
+    );
+    /*!
+        ensures
+            - returns the result of adding value to all the elements of matrix m.  
+              The resulting matrix will have the same dimensions as m.
+    !*/
+
+    template <typename T>
+    const matrix_exp operator- (
+        const matrix_exp& m,
+        const T& value
+    );
+    /*!
+        ensures
+            - returns the result of subtracting value from all the elements of matrix m.  
+              The resulting matrix will have the same dimensions as m.
+    !*/
+
+    template <typename T>
+    const matrix_exp operator- (
+        const T& value,
+        const matrix_exp& m
+    );
+    /*!
+        ensures
+            - Returns a matrix M such that:
+                - M has the same dimensions as m
+                - M contains the same type of element as m
+                - for all valid r and c:
+                    - M(r,c) == value - m(r,c)
+    !*/
+
     bool operator== (
         const matrix_exp& m1,
         const matrix_exp& m2
@@ -136,7 +194,7 @@ namespace dlib
         typename T,
         long num_rows = 0,
         long num_cols = 0,
-        typename mem_manager = memory_manager<char>::kernel_1a,
+        typename mem_manager = default_memory_manager,
         typename layout = row_major_layout 
         >
     class matrix : public matrix_exp<matrix<T,num_rows,num_cols,mem_manager,layout> > 
@@ -457,6 +515,22 @@ namespace dlib
                 - returns *this
         !*/
 
+        template <typename EXP>
+        matrix& operator *= (
+            const matrix_exp<EXP>& m
+        );
+        /*!
+            requires
+                - matrix_exp<EXP>::type == T
+                  (i.e. m must contain the same type of element as *this)
+                - nc() == m.nr()
+                - size() > 0 && m.size() > 0
+                  (you can't multiply any sort of empty matrices together)
+            ensures
+                - #(*this) == *this * m
+                - returns *this
+        !*/
+
         matrix& operator *= (
             const T& a
         );
@@ -475,6 +549,24 @@ namespace dlib
                 - returns *this
         !*/
 
+        matrix& operator += (
+            const T& a
+        );
+        /*!
+            ensures
+                - #(*this) == *this + a
+                - returns *this
+        !*/
+
+        matrix& operator -= (
+            const T& a
+        );
+        /*!
+            ensures
+                - #(*this) == *this - a
+                - returns *this
+        !*/
+
         const literal_assign_helper operator = (
             const T& val
         );
