@@ -34,8 +34,8 @@ namespace dlib
             verbose = false;
             ekm_stale = true;
 
-            initial_basis_size = 5;
-            basis_size_increment = 10;
+            initial_basis_size = 10;
+            basis_size_increment = 50;
             max_basis_size = 300;
         }
 
@@ -56,8 +56,8 @@ namespace dlib
             verbose = false;
             ekm_stale = true;
 
-            initial_basis_size = 5;
-            basis_size_increment = 10;
+            initial_basis_size = 10;
+            basis_size_increment = 50;
             max_basis_size = 300;
         }
 
@@ -82,6 +82,19 @@ namespace dlib
             return ocas.get_epsilon();
         }
 
+        void set_max_iterations (
+            unsigned long max_iter
+        )
+        {
+            ocas.set_max_iterations(max_iter);
+        }
+
+        unsigned long get_max_iterations (
+        )
+        {
+            return ocas.get_max_iterations();
+        }
+
         void be_verbose (
         ) 
         { 
@@ -422,10 +435,10 @@ namespace dlib
             // we will use a linearly_independent_subset_finder to store our basis set. 
             linearly_independent_subset_finder<kernel_type> lisf(get_kernel(), max_basis_size);
 
-            dlib::rand::kernel_1a rnd;
+            dlib::rand rnd;
 
             // first pick the initial basis set randomly
-            for (unsigned long i = 0; i < 10*initial_basis_size && lisf.dictionary_size() < initial_basis_size; ++i)
+            for (unsigned long i = 0; i < 10*initial_basis_size && lisf.size() < initial_basis_size; ++i)
             {
                 lisf.add(x(rnd.get_random_32bit_number()%x.size()));
             }
@@ -458,7 +471,7 @@ namespace dlib
             {
                 // if the basis is already as big as it's going to get then just do the most
                 // accurate training right now.  
-                if (lisf.dictionary_size() == max_basis_size)
+                if (lisf.size() == max_basis_size)
                     trainer.set_epsilon(min_epsilon);
 
                 while (true)
@@ -484,7 +497,7 @@ namespace dlib
                 if (verbose)
                 {
                     std::cout << "svm objective: " << svm_objective << std::endl;
-                    std::cout << "basis size: " << lisf.dictionary_size() << std::endl;
+                    std::cout << "basis size: " << lisf.size() << std::endl;
                 }
 
                 // if we failed to make progress on this iteration then we are done
@@ -496,7 +509,7 @@ namespace dlib
                 // now add more elements to the basis
                 unsigned long count = 0;
                 for (unsigned long j = 0; 
-                     (j < 100*basis_size_increment) && (count < basis_size_increment) && (lisf.dictionary_size() < max_basis_size); 
+                     (j < 100*basis_size_increment) && (count < basis_size_increment) && (lisf.size() < max_basis_size); 
                      ++j)
                 {
                     // pick a random sample
