@@ -35,10 +35,16 @@ public:
 	void SetParams(float strength, bool adaptive);
 
 	void Draw(QPainter &painter);
+    std::vector<GLObject> DrawGL();
 	void Train(float *dataMap, fVec size, fvec startingPoint=fvec());
 	fvec Test( const fvec &sample);
 	fvec Test(const fVec &sample);
-	char *GetInfoString();
+    const char *GetInfoString();
+    void SetParams(Maximizer *maximizer, fvec parameters);
+    fvec GetParams();
+    void GetParameterList(std::vector<QString> &parameterNames,
+                                 std::vector<QString> &parameterTypes,
+                                 std::vector< std::vector<QString> > &parameterValues);
 };
 
 #endif // _MAXIMIZE_GRADIENT_H_
