@@ -14,13 +14,15 @@ private:
     QWidget *widget;
 public:
     LDAProjection();
+    ~LDAProjection();
     // virtual functions to manage the algorithm creation
     Projector *GetProjector();
     void DrawInfo(Canvas *canvas, QPainter &painter, Projector *projector);
     void DrawModel(Canvas *canvas, QPainter &painter, Projector *projector);
+    void DrawGL(Canvas *canvas, GLWidget *glw, Projector *projector){}
 
     // virtual functions to manage the GUI and I/O
-    QString GetName(){return QString("LDA");}
+    QString GetName(){return QString("Linear Discriminant Analysis");}
     QString GetAlgoString();
     QString GetInfoFile(){return "LDA.html";}
     QWidget *GetParameterWidget(){return widget;}
@@ -29,6 +31,11 @@ public:
     bool LoadOptions(QSettings &settings);
     void SaveParams(QTextStream &stream);
     bool LoadParams(QString name, float value);
+    void SetParams(Projector *projector, fvec parameters);
+    fvec GetParams();
+    void GetParameterList(std::vector<QString> &parameterNames,
+                                 std::vector<QString> &parameterTypes,
+                                 std::vector< std::vector<QString> > &parameterValues);
 };
 
 #endif // INTERFACELDAPROJECTION_H
