@@ -22,16 +22,18 @@ Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 #include "interfacePCAProjection.h"
 #include "interfaceLDAProjection.h"
 #include "interfaceKPCAProjection.h"
+#include "interfaceSammonProjection.h"
 
 using namespace std;
 
 PluginProjections::PluginProjections()
 {
-    //classifiers.push_back(new ClassProjections());
+//    classifiers.push_back(new ClassProjections());
     projectors.push_back(new ICAProjection());
     projectors.push_back(new PCAProjection());
     projectors.push_back(new LDAProjection());
     projectors.push_back(new KPCAProjection());
+    projectors.push_back(new SammonProjection());
 }
 
 Q_EXPORT_PLUGIN2(mld_Projections, PluginProjections)
